@@ -88,7 +88,7 @@ static void sig_handler(int signum, siginfo_t *si, void *uc)
 {
 #if defined(__powerpc__) || defined(__powerpc64__) || defined(__ia64__) || \
     defined(__s390__) || defined(__s390x__) || defined(__sparc__) || \
-    defined(__aarch64__) || (defined(__riscv) && __riscv_xlen == 64)
+    defined(__aarch64__) || (defined(__riscv) && __riscv_xlen == 64 || __loongarch64)
 	/* On powerpc, ia64, s390 and Aarch64, 0 bytes are an illegal
 	 * instruction, so, if the icache is cleared properly, we SIGILL
 	 * as soon as we jump into the cleared page */
